#include "headers.h"

MODULE = Arisa::Interface  PACKAGE = Arisa
PROTOTYPES: ENABLE

Arisa::Interface
interface_find(name = NULL)
	char *name
CODE:
	RETVAL = interface_find(name);
OUTPUT:
	RETVAL

void
interfaces()
PREINIT:
	int i;
PPCODE:
	LOCK(global);
	for(i = 0; i < global->no_interfaces; ++i) {
		interface_t *intf = global->interfaces[i];
		LOCK(intf);
		if(intf->deleted == 0)
			XPUSHs(sv_2mortal(
				create_ptr_object("Arisa::Interface",intf)));
		UNLOCK(intf);
	}
	UNLOCK(global);

Arisa::Interface
interface_add(name = NULL, typestr = "SEND")
	char *name
	char *typestr
PREINIT:
	int type;
CODE:
	type = str_to_interfacetype(typestr);
	RETVAL = NULL;
	if(name != NULL && type != -1) {
		interface_t *intf = alloc_interface(type);
		
		intf->name = xstrdup(name);
		
		if(interface_add(intf) == 0)
			RETVAL = intf;
		else
			free_interface(intf);
	}
OUTPUT:
	RETVAL

	
#*******************************
MODULE = Arisa::Interface  PACKAGE = Arisa::Interface
#*******************************

SV*
access_add(intf, ...)
	Arisa::Interface intf
CODE:
	RETVAL = &PL_sv_no;
	if(interface_valid(intf)) {
		ACCESS_ADD(intf,intf->recv.access,
				intf->type == INTERFACE_RECV);
	}
OUTPUT:
	RETVAL

SV*
access_check(intf, ...)
	Arisa::Interface intf
CODE:
	RETVAL = &PL_sv_no;
	if(interface_valid(intf)) {
		ACCESS_CHECK(intf,intf->recv.access,
				intf->type == INTERFACE_RECV);
	}
OUTPUT:
	RETVAL

SV*
access_del(intf, ...)
	Arisa::Interface intf
CODE:
	RETVAL = &PL_sv_no;
	if(interface_valid(intf)) {
		ACCESS_DEL(intf,intf->recv.access,
				intf->type == INTERFACE_RECV);
	}
OUTPUT:
	RETVAL
	
void
access_list(intf)
	Arisa::Interface intf
PPCODE:
	if(interface_valid(intf)) {
		ACCESS_LIST(intf,intf->recv.access,
				intf->type == INTERFACE_RECV);
	}

SV*
access_mode(intf, ...)
	Arisa::Interface intf
CODE:
	RETVAL = &PL_sv_undef;
	if(interface_valid(intf)) {
		ACCESS_MODE(intf,intf->recv.access,
				intf->type == INTERFACE_RECV);
	}
OUTPUT:
	RETVAL

void
chats(intf)
	Arisa::Interface intf
PPCODE:
	if(interface_valid(intf)) {
		LOCK(intf);
		if(intf->type == INTERFACE_CHAT) {
			int i;
			for(i = 0; i < intf->chat.no_chats; ++i) {
				LOCK(intf->chat.chats[i]);
				XPUSHs(sv_2mortal(newRV_noinc(
					(SV*)chat_to_hv(intf->chat.chats[i]))));
				UNLOCK(intf->chat.chats[i]);
			}
		}
		UNLOCK(intf);
	}

int
close_chat(intf, ...)
	Arisa::Interface intf
CODE:
	RETVAL = 0;
	if(interface_valid(intf)) {
		interfacetype_t type;
		LOCK(intf);
		type = intf->type;
		UNLOCK(intf);
		if(type == INTERFACE_CHAT) {
			network_t *net	= SvPTRarg("network");
			char *host	= SvPVarg("host");
			char *nick	= SvPVarg("nick");
			char *reason	= SvPVarg("reason");
			void *ptr	= (void *)SvIVarg("_ptr");

			RETVAL = interface_chat_close(intf,host,net,
					nick,ptr,reason);
		}
	}
OUTPUT:
	RETVAL

int
close_upload(intf, ...)
	Arisa::Interface intf
CODE:
	RETVAL = -1;
	if(interface_valid(intf)) {
		interfacetype_t type;
		LOCK(intf);
		type = intf->type;
		UNLOCK(intf);
		if(type == INTERFACE_RECV) {
			network_t *net	= SvPTRarg("network");
			char *host	= SvPVarg("host");
			char *nick	= SvPVarg("nick");
			char *file	= SvPVarg("file");
			char *reason	= SvPVarg("reason");
			void *ptr	= (void *)SvIVarg("_ptr");

			RETVAL = interface_upload_close(intf,host,net,
					nick,file,ptr,reason);
		}
	}
OUTPUT:
	RETVAL

SV*
delete(intf)
	Arisa::Interface intf
CODE:
	RETVAL = &PL_sv_no;
	if(interface_valid(intf)) {
		if(interface_del(intf) == 0)
			RETVAL = &PL_sv_yes;
	}
OUTPUT:
	RETVAL

SV*
name(intf, new_name = NULL)
	Arisa::Interface intf
	char *new_name
CODE:
	RETVAL = &PL_sv_no;
	if(interface_valid(intf)) {
		if(new_name != NULL) {
			if(interface_rename(intf,new_name) == 0)
				RETVAL = &PL_sv_yes;
		} else {
			LOCK(intf);
			RETVAL = new_pv(intf->name);
			UNLOCK(intf);
		}
	}
OUTPUT:
	RETVAL

void
info(intf)
	Arisa::Interface intf
PREINIT:
	HV *hv = NULL;
PPCODE:
	if(interface_valid(intf)) {
		AV *avgspeed = newAV();
		uint64_t rate; // incase we have a *really* fast interface
		int i;
		
		hv = newHV();
		
		LOCK(intf);
		hv_store(hv,"type",4,
			new_pv(interfacetype_to_str(intf->type)),0);
		hv_store(hv,"default",7,
			intf->def != 0 ? &PL_sv_yes : & PL_sv_no,0);
		for(i = 0, rate = 0; i < AVGSPEED; ++i) {
			rate += intf->avgspeed[i];
			av_push(avgspeed,newSVuv(intf->avgspeed[i]));
		}
		hv_store(hv,"rate",4,
			newSVuv((unsigned int)(rate / (uint64_t)AVGSPEED)),0);
		hv_store(hv,"rate_data",9,newRV_noinc((SV*)avgspeed),0);
		hv_store(hv,"record_rate",11,newSVuv(intf->record_rate),0);
		hv_store(hv,"total_transferred",17,newSVnv(
					((double)intf->stat_mib)*(1024.0*1024.0)
					+ (double)intf->stat_carry),0);
		UNLOCK(intf);
	}
	RET_REF(hv);

void
reset_counters(intf)
	Arisa::Interface intf
PPCODE:
	if(interface_valid(intf)) {
		if(interface_reset_counters(intf) == 0)
			XPUSHs(&PL_sv_yes);
		else
			XPUSHs(&PL_sv_no);
	}

void
set_default(intf)
	Arisa::Interface intf
PPCODE:
	if(interface_valid(intf)) {
		if(interface_set_default(intf) == 0)
			XPUSHs(&PL_sv_yes);
		else
			XPUSHs(&PL_sv_no);
	}
	
void
settings(intf, ...)
	Arisa::Interface intf
PREINIT:
	HV *hv = NULL;
PPCODE:
	if(interface_valid(intf)) {
		hv = newHV();
		perl_apply_settings(interface_apply_setting,intf);
		perl_read_settings(interface_read_settings,intf,hv);
	}
	RET_REF(hv);

void
uploads(intf)
	Arisa::Interface intf
PPCODE:
	if(interface_valid(intf)) {
		LOCK(intf);
		if(intf->type == INTERFACE_RECV) {
			int i;
			for(i = 0; i < intf->recv.no_uploads; ++i) {
				LOCK(intf->recv.uploads[i]);
				XPUSHs(sv_2mortal(newRV_noinc(
					(SV*)send_to_hv(intf->recv.uploads[i]))));
				UNLOCK(intf->recv.uploads[i]);
			}
		}
		UNLOCK(intf);
	}
